﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.AccountDetail.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.AccountDetail',
    xtype: 'App.Control.Kivii.Finances.AccountDetail.List',
    viewModel: {
        data: {
            _selection: null,
            _owner: {}
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 200,
        sorters: [{
            property: 'CreateTime',
            direction: 'DESC'
        }]
    },
    features: [{
        ftype: 'summary'
    }],
    tbar: [{
            xtype: 'searchField',
            flex: 1,
            labelWidth: 60,
            queryField: 'PayerName,PayeeName,Remark',
            emptyText: '模糊搜索收付款单位'
        },{
        margin: '0 0 -9 10',
        xtype: 'displayfield',
        bind: {
            value: '账号名称:{_owner.OwnerName} ({_owner.Name})'
        },
        fieldStyle: {
            fontSize: '16px'
        }
    }],
    columnLines: true,
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center'
        }, {
            xtype: 'datecolumn',
            text: '交易日期',
            format: 'Y-m-d H:i',
            dataIndex: 'CreateTime',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            width: 150,
            // renderer: '_renderPayerName'
        },{
            text: '收款单位',
            dataIndex: 'PayeeName',
            width: 150,
            // renderer: '_renderPayerName'
        }, {
            text: '借',
            dataIndex: 'AmountPayment',
            align: 'right',
            renderer: '_renderDebit',
            width: 120,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            }
        }, {
            text: '贷',
            dataIndex: 'AmountPayment',
            align: 'right',
            renderer: '_renderCredit',
            width: 120,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            }
        }, {
            text: '帐户余额',
            dataIndex: 'Amount',
            align: 'right',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 120
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 200
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 100,
            flex: 1
        }]
    },
    setOwner: function(_owner) {
        let _view = this;
        let _vm = _view.getViewModel();
        _vm.set('_owner', _owner);
    },
    controller: {
    	//我增加
        _renderDebit: function(_value, _meta, _record, _rowIndex) {
            if(_record.data.PayeeAccountKvid===_record.data.AccountKvid){
            	return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
            }else {
            	return 0.00;
            }
        },
        //我减少
        _renderCredit: function(_value, _meta, _record, _rowIndex) {
            if(_record.data.PayerAccountKvid===_record.data.AccountKvid){
            	return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
            }else {
            	return 0.00;
            }
        },
        _renderPayerName: function(_value, _meta, _record, _rowIndex) {
        	let _controller=this;
        	let _vm=_controller.getViewModel();
        	if(_vm.data._owner.Type!=='Balance'&&_vm.data._owner.Type!=='Discount'){
	            if(_record.data.PayerAccountKvid===_record.data.AccountKvid){
	            	return _record.data.PayeeName;
	            }else if(_record.data.PayeeAccountKvid===_record.data.AccountKvid){
	            	return _record.data.PayerName;
	            }else{
	            	return '未知问题';
	            }
        	}
        	else{
        		if(_record.data.PayeeAccountKvid===_record.data.AccountKvid){
	            	return _record.data.PayeeName;
	            }else if(_record.data.PayerAccountKvid===_record.data.AccountKvid){
	            	return _record.data.PayerName;
	            }else{
	            	return '未知问题';
	            }
        	}
        }
    }
});