﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Bill.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Bill',
    xtype: 'App.View.Kivii.Finances.Bill.List',
    title: '账单管理',
    iconCls: 'x-fa fa-file-text-o',
    filter: {
    	OwnerKvid:KiviiContext.CurrentMember.DepartmentKvid,
        IsPayed: false
    },
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 400,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'groupingsummary',
        groupHeaderTpl: [
            '<div>{name:this.formatName} ({rows.length} 条)</div>',
            {
                formatName: function(name) {
                    return name;
                }
            }
        ]
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'isPayed',
                value: false,
                editable: false,
                filterPickList: true,
                reference: 'isPayed',
                store: {
                    fields: ['displayInfo', 'isPayed'],
                    data: [{
                        displayInfo: '全部',
                        isPayed: null
                    }, {
                        displayInfo: '已收款',
                        isPayed: true
                    }, {
                        displayInfo: '未收款',
                        isPayed: false
                    }]
                },
                listeners: {
                    select: '_onSelectIsPayed'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '时间范围 开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanBeginOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanEndOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'PayerName,OperatorName,OwnerName',
                emptyText: '模糊搜索付款单位、操作人',
            }, {
                text: '创建账单',
                iconCls: 'x-fa fa-file-text-o',
                handler: '_onGenerateBill'
            }, {
                text: '删除',
                iconCls: 'x-fa fa-trash-o',
                iconAlign: 'left',
                bind: {
                    disabled: '{!_selection}'
                },
                handler: '_onDeleteEntity'
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer'
        }, {
            dataIndex: 'OwnerName',
            text: '所属部门',
            width: 280,
            // sortable: true,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('合计');
            }
        }, {
            dataIndex: 'PayerName',
            text: '付款单位',
            width: 300
        }, {
            dataIndex: 'OperateTime',
            text: '操作日期',
            width: 100,
            // sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'Amount',
            align: 'right',
            text: '账单金额',
            width: 100,
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'AmountPayment',
            align: 'right',
            text: '已付金额',
            renderer: '_renderAmount',
            width: 100,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'OperatorName',
            text: '操作人',
            width: 90
        }, {
            xtype: 'actioncolumn',
            text: '操作',
            align: 'center',
            items: [{
                text: '查看',
                iconCls: 'x-fa fa-search',
                handler: '_onReadBill'
            }],
            width: 50
        }, {
            dataIndex: 'Summary',
            text: '摘要',
            width: 200
        }, {
            dataIndex: 'Remark',
            text: '备注',
            minWidth: 100,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            } else _meta.style = 'background-color:#F0E68C';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },

        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            _view.reload();
        },
        _cleanEndOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeLessThan;
            _view.reload();
        },
        _onSelectIsPayed: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.IsPayed = _record.data.isPayed;
            _view.reload();
        },
        _onDeleteEntity: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            _view.deleteEntity();
        },
        _onGenerateBill: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _form = Ext.create('App.Control.Kivii.Finances.Bill.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 700,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onReadBill: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.Bill.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 680,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_record.data);
            _form.on('entityDeleted', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
            _win.on('close', function() {
                _view.reload();
            });
            // Ext.Ajax.request({
            //     url: '/Restful/Kivii.Finances.Entities.Settlement/Query.json',
            //     params: {
            //         BillKvid: _record.data.Kvid
            //     },
            //     success: function(_response) {
            //         let _rtns = Ext.decode(_response.responseText);
            //         let _form = Ext.create('App.Control.Kivii.Finances.Bill.Form', {
            //             header: false
            //         });
            //         let _win = Ext.create('Ext.window.Window', {
            //             layout: 'fit',
            //             title: _ctrl.text,
            //             iconCls: _ctrl.iconCls,
            //             iconAlign: 'left',
            //             // header:false,
            //             autoScroll: true,
            //             width: 1200,
            //             height: 680,
            //             modal: true,
            //             maximizable: true,
            //             resizable: true,
            //             items: [_form]
            //         });
            //         _win.show();
            //         _form.setOwner(_record.data,_rtns.Results);
            //         _form.on('entityDeleted', function(_ctrl, _entity) {
            //             _win.close();
            //             _view.reload();
            //         });
            //         _win.on('close',function(){
            //         	_view.reload();	
            //         });
            //     },
            //     failure: function(_response) {
            //         Ext.toast('查看失败,请重试!');
            //     }
            // });

        }
    }
});