﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Payment.Biz.ErpPayment.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.View.Kivii.Finances.Payment.Biz.ErpPayment.List',
    dataSource: '/Restful/Njzj.Connects.Entities.NjzjPayment/Query.json',
    title: 'ERP银行流水',
    iconCls: 'x-fa fa-cny',
    filter: {
        IsSerial: true
        // IncludeRepeat:true
    },
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 400
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,PayerAccountName,PayerAccountSerialNumber',
                emptyText: '模糊搜索流水号、付款单位、收付款账号或名称',
            }, {
                text: '银行到账',
                iconCls: 'x-fa fa-bank',
                type: 'Bank',
                bind: {
                    disabled: '{!_selection}'
                },
                handler: '_onAccept'
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '所属范围',
            dataIndex: 'OwnerName',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return '合计 ￥';
            },
            width: 220
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 110
        },{
            xtype: 'datecolumn',
            text: '发生日期',
            format: 'Y-m-d H:i',
            dataIndex: 'OperateTime',
            width: 150
        }, {
            text: '付款类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            text: '付款银行',
            dataIndex: 'PayerAccountName',
            width: 280
        }, {
            text: '付款账号',
            dataIndex: 'PayerAccountSerialNumber',
            width: 180
        }, {
            text: '备注',
            dataIndex: 'Remark',
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === 'Cash') {
                return '现金';
            }
            if (_value === 'Pos') {
                return '刷卡';
            }
            if (_value === 'AliPay') {
                return '支付宝';
            }
            if (_value === 'WeChat') {
                return '微信';
            }
            if (_value === 'Bank') {
                return '银行转账';
            }
            return _value;
        },

        _onAccept: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            if (!_view.selection) return;
            let _item = {};
            Ext.apply(_item,_view.selection.data);
            _item.Kvid = null;
            _item.PayeeName = KiviiContext.CurrentMember.OrganizationName;
            _item.OperateTime=new Date(_item.OperateTime);

            let _form = Ext.create('App.Control.Kivii.Finances.Payment.Accept.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 850,
                height: 410,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _form.setOwner(_item, _ctrl);
            _win.show();
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        }
    }
});