﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Statistic.Invoice', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Statistic.Invoice',
    entityType: 'Kivii.Finances.Entities.Invoice',
    dataSource: '/Restful/Kivii.Finances.Entities.Invoice/Statistic.json',
    globalFilter: {},
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        Offseted:false,
        Payed: true
    },
    header: true,
    layout: 'fit',
    title: '发票明细统计',
    iconCls: 'x-fa fa-money',
    viewModel: {
        data: {
            amount: null,
            amountUntaxed: null,
            amountPayment: null,
            _selection: null,
            lastOwnerKvid: null
        },
        formulas: {
            allowOffset: function(get) {
                let _selection = get('_selection');
                if (!_selection) return false;
                if (_selection.data.OffsetKvid) return false;
                if (_selection.data.BizKvid) return false;
                return true;
            },
            allowSplit: function(get) {
                let _selection = get('_selection');
                if (!_selection) return false;
                if (_selection.data.ParentKvid) return false;
                if (_selection.data.AmountUsed == _selection.data.Amount) return false;
                return true;
            },
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'OperateTime',
            direction: 'ASC'
        }]
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
            xtype: 'combo',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'offseted',
            value: false,
            editable: false,
            filterPickList: true,
            reference: 'offseted',
            store: {
                fields: ['name', 'offseted'],
                data: [{
                    name: '已开发票',
                    offseted: false
                },  {
                    name: '作废及红字',
                    offseted: true
                }]
            },
            listeners: {
                select: '_onSelectType'
            }
        }, {
            xtype: 'combo',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'payed',
            value: true,
            emptyText: '--仅针对全部或借票有效--',
            editable: false,
            readOnly: false,
            filterPickList: true,
            reference: 'payed',
            store: {
                fields: ['name', 'payed'],
                data: [{
                    name: '已到账',
                    payed: true
                }, {
                    name: '未到账',
                    payed: false
                    // }, {
                    //     name: '全部',
                    //     payed: ''
                }]
            },
            listeners: {
                select: '_onSelectPayed'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '时间范围  开始',
            format: 'Y-m-d',
            labelWidth: 90,
            reference: 'startTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '结束',
            format: 'Y-m-d',
            labelWidth: 30,
            reference: 'endTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth(), 0),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'searchField',
            flex: 1,
            labelWidth: 60,
            queryField: 'OwnerName,SerialNumber,PayerName',
            emptyText: '模糊搜索部门、发票编号、发票抬头'
        }, {
            text: '导出报表',
            iconCls: 'x-fa fa-download',
            handler: '_onDownloadExcel'
        }]
    }, {
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            margin: '0 0 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '全页 开票金额: {amount} &nbsp;&nbsp;已到账: {amountPayment} &nbsp;&nbsp;<font color="red">未到账: {amountUntaxed} </font>'
            },
            fieldStyle: {
                fontSize: '14px'
            }
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center'
        }, {
            text: '发票编码',
            dataIndex: 'SerialNumber',
            width: 90
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 150
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return '当页小计 ￥';
            },
            width: 200
        }, {
            xtype: 'datecolumn',
            text: '开票日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 110
        }, {
            text: '开票金额(含税)',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 130
        }, {
            text: '收入金额(不含税)',
            align: 'right',
            dataIndex: 'AmountUntaxed',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 140
        }, {
            text: '到账金额',
            align: 'right',
            dataIndex: 'AmountPayment',
            renderer: '_renderAmountPayment',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 130
        }, {
            xtype: 'datecolumn',
            text: '到账日期',
            format: 'Y-m-d',
            dataIndex: 'PayedTime',
            width: 110
        }, {
            text: '申请人',
            dataIndex: 'OperatorName',
            width: 90
        // }, {
        //     xtype: 'actioncolumn',
        //     text: '分摊状态',
        //     align: 'center',
        //     items: [{
        //         iconCls: 'x-fa fa-search',
        //         handler: '_onPayed'
        //     }],
        //     width: 90
        },{
            dataIndex: 'Summary',
            text: '摘要',
            sortable: false,
            width: 230
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 100,
            sortable: false,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate',
        viewready: '_viewReady'
    },
    controller: {
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _viewReady: function() {
            let _me = this;
            let _view = _me.getView();
            let _vm = _me.getViewModel();
            let _store = _view.getStore();
            _store.on('load', function(_this, _records, _successful, _operation, _eOpts) {
            	let _rtns = Ext.decode(_operation.getResponse().responseText);
            	_vm.set('amount', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_rtns.TotalAmount.toFixed(2)));
                _vm.set('amountPayment', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_rtns.TotalAmountPayment.toFixed(2)));
                _vm.set('amountUntaxed', KiviiContext.Const.FinancesFunctionsMoneyToFmoney((_rtns.TotalAmount-_rtns.TotalAmountPayment).toFixed(2)));
            });
        },
        _renderPayerCompanyAddress: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerPhone;
        },
        _renderPayerRegisteredBank: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerBankAccount;
        },
        _renderAmountPayment: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.AmountPayment > 0 && _record.data.AmountPayment < _record.data.Amount) {
                _meta.style = 'background-color:#4682B4';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _onSelectType: function(_ctrl, _record) {
            let _controller = this,
                _view = _controller.getView(),
                _vm = _controller.getViewModel(),
                _refs = _controller.getReferences();
            delete _view.filter.Offseted;
            _view.filter.Offseted = _record.data.offseted;
            let _payed = _refs.payed;
            if (!_record.data.offseted) {
                _payed.setReadOnly(false);
                _payed.setValue(true);
                _view.filter.Payed = true;
            } else {
                _payed.setReadOnly(true);
                _payed.setValue(null);
                delete _view.filter.Payed;
            }
            _view.reload();
        },
        _onSelectPayed: function(_ctrl, _record) {
            let _controller = this,
                _view = _controller.getView(),
                _vm = _controller.getViewModel();
            delete _view.filter.Payed;
            _view.filter.Payed = _record.data.payed;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _onDownloadExcel: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _filter = _view.getFilter();
            let _globalFilter = _view.getGlobalFilter();
            let _searchFilter = _view.getSearchFilter();
            let _selector = Ext.create('App.Control.Kivii.Finances.General.Template.Selector');

            let _year = _refs.startTime.value.getFullYear();
            let _month = _refs.startTime.value.getMonth() + 1;

            let _offseted = _refs.offseted.getDisplayValue();
            let _name = '[' + _year + '年' + _month + '月份]' + _offseted;
            let _payed = _refs.payed.getDisplayValue();
            if (_payed !== "") _name += '_' + _payed;

            let _pageSize = _view.getStore().getPageSize();
            let _currentPage = _view.getStore().currentPage;
            let _totalPage = Math.ceil(_view.getStore().totalCount / _pageSize);

            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: '选择模板',
                iconCls: 'x-fa fa-money',
                iconAlign: 'left',
                autoScroll: true,
                width: 700,
                height: 350,
                modal: true,
                resizable: false,
                items: [_selector]
            });
            _win.show();
            _selector.setFilter({
            	FolderPath: '/Finances/Invoice/Files',
                Tags: 'Invoice'
            });
            _selector.on('entitySelected', function(_ctrl, _entity) {
                _win.close();
                let _iframe = document.createElement("iframe");
                _iframe.style.display = "none";
                let _form = document.createElement('form');
                for (let _globalKey in _globalFilter) {
                    let _globalValue = document.createElement('input');
                    _globalValue.type = 'hidden';
                    _globalValue.name = _globalKey;
                    _globalValue.value = _globalFilter[_globalKey];
                    if ((typeof _globalFilter[_globalKey] === 'object') && _globalFilter[_globalKey].constructor === Date) {
                        let _globalDate = _globalFilter[_globalKey].getFullYear() + '-' + (_globalFilter[_globalKey].getMonth() + 1) + '-' + _globalFilter[_globalKey].getDate();
                        _globalValue.value = _globalDate;
                    }
                    _form.appendChild(_globalValue);
                }
                for (let _filterKey in _filter) {
                    let _filterValue = document.createElement('input');
                    _filterValue.type = 'hidden';
                    _filterValue.name = _filterKey;
                    _filterValue.value = _filter[_filterKey];
                    if ((typeof _filter[_filterKey] === 'object') && _filter[_filterKey].constructor === Date) {
                        let _filterDate = _filter[_filterKey].getFullYear() + '-' + (_filter[_filterKey].getMonth() + 1) + '-' + _filter[_filterKey].getDate();
                        _filterValue.value = _filterDate;
                    }
                    _form.appendChild(_filterValue);
                }
                for (let _searchKey in _searchFilter) {
                    let _searchValue = document.createElement('input');
                    _searchValue.type = 'hidden';
                    _searchValue.name = _searchKey;
                    _searchValue.value = _searchFilter[_searchKey];
                    if ((typeof _searchFilter[_searchKey] === 'object') && _searchFilter[_searchKey].constructor === Date) {
                        let _searchDate = _searchFilter[_searchKey].getFullYear() + '-' + (_searchFilter[_searchKey].getMonth() + 1) + '-' + _searchFilter[_searchKey].getDate();
                        _searchValue.value = _searchDate;
                    }
                    _form.appendChild(_searchValue);
                }

                let _view = document.createElement('input');
                _view.type = 'hidden';
                _view.name = 'View';
                _view.value = _entity[0].Path;
                _form.appendChild(_view);

                let _fileName = document.createElement('input');
                _fileName.type = 'hidden';
                _fileName.name = '__fileName';
                _fileName.value = _name + '(第' + _currentPage + '页,共' + _totalPage + '页)';
                _form.appendChild(_fileName);

                let _skip = document.createElement('input');
                _skip.type = 'hidden';
                _skip.name = 'Skip';
                _skip.value = (_currentPage - 1) * _pageSize;
                _form.appendChild(_skip);

                let _take = document.createElement('input');
                _take.type = 'hidden';
                _take.name = 'Take';
                _take.value = _pageSize;
                _form.appendChild(_take);

                let _orderBy = document.createElement('input');
                _orderBy.type = 'hidden';
                _orderBy.name = 'OrderBy';
                _orderBy.value = 'OperateTime';
                _form.appendChild(_orderBy);

                _iframe.appendChild(_form);
                document.body.appendChild(_iframe);
                _form.method = 'GET';
                _form.action = '/Restful/Kivii.Finances.Entities.Invoice/Statistic.xlsx';
                _form.submit();
                document.body.removeChild(_iframe);
            });

        }
    }
});