﻿using Kivii.Finances.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class InvoiceExtension
    {
        public static Invoice Offset(this Invoice invoice, string remark)
        {
            invoice.ThrowIfNull("发票不能为空!");
            (invoice.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (invoice.OffsetKvid != Guid.Empty).ThrowIfTrue("发票已作废,无需重复操作!");

            #region invoice冲账
            var newInvoiceKvid = Guid.NewGuid();
            invoice.OffsetKvid = newInvoiceKvid;
            invoice.AddOnlyProperties(o => o.OffsetKvid);
            invoice.Remark = $"{remark}";
            invoice.AddOnlyProperties(o => o.Remark);

            //新建一个和原来一样的
            var offsetInvoice = new Invoice();
            offsetInvoice.PopulateWith(invoice);
            offsetInvoice.OffsetKvid = invoice.Kvid;//新的冲帐关联用原来的Kvid
            offsetInvoice.BizId = string.Empty;
            offsetInvoice.BizKvid = Guid.Empty;
            offsetInvoice.BizType = string.Empty;
            offsetInvoice.Kvid = newInvoiceKvid;
            offsetInvoice.RootKvid = newInvoiceKvid;
            offsetInvoice.ParentKvid = Guid.Empty;
            offsetInvoice.SerialNumber = invoice.SerialNumber.IsNullOrEmpty() ? "" : ("-" + invoice.SerialNumber);
            offsetInvoice.Remark = $"{remark}";
            offsetInvoice.Amount = 0 - offsetInvoice.Amount;//反向值
            offsetInvoice.AmountPayment = 0 - offsetInvoice.AmountPayment;
            offsetInvoice.OperateTime = DateTime.Now;
            #endregion

            return offsetInvoice;
        }
    }
}
