﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "结算作废")]//区别于 删除 当此结算已经收款登记时无法删除 只能作废,相反同理
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementOffset: RestfulExecutionGeneric<Settlement>
    {
        public List<Guid> Kvids { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("参数错误:未传入要作废的结算Kvids");

            var connF = KiviiContext.GetOpenedDbConnection<Settlement>();
            //var connE = KiviiContext.GetOpenedDbConnection<EntitySettlement<G>>();
            var settlements = connF.Select<Settlement>(o => Sql.In(o.Kvid, Kvids));
            (settlements.Count != Kvids.Count).ThrowIfTrue("所选结算和查询结果不一致!");
            settlements.Exists(o => o.OffsetKvid != Guid.Empty).ThrowIfTrue("存在已作废结算,无法重复作废");
            settlements.Exists(o => o.AmountPayment != o.Amount).ThrowIfTrue("作废错误:作废的结算中包含未收款的结算!");
            //connF.Exists<Settlement>(o => o.AmountPayment != o.Amount && Sql.In(o.Kvid, Kvids)).ThrowIfTrue("作废错误:作废的结算中包含未收款的结算!");
            ////如果泛型类型不是Settlement本身就需要判断
            //if (typeof(G) != typeof(Settlement)) connE.Exists<EntitySettlement<G>>(o => o.AmountPayment != o.Amount && Sql.In(o.Kvid, Kvids)).ThrowIfTrue("作废错误:作废的结算中包含未收款的结算!");
            List<Payment> allPayments = new List<Payment>();
            List<Account> allFromAccounts = new List<Account>();
            List<Account> allToAccounts = new List<Account>();

            List<Payment> paymentsPreUpdate = new List<Payment>();
            List<Payment> parentsPreUpdate = new List<Payment>();
            List<AccountDetail> accountDetailsPreInsert = new List<AccountDetail>();
            List<Pay> paysOffsetPreUpdate = new List<Pay>();
            List<Pay> paysOffsetPreInsert = new List<Pay>();
            List<Settlement> settlementsOffsetPreUpdate = new List<Settlement>();
            List<Settlement> settlementsOffsetPreInsert = new List<Settlement>();

            var queryPays =connF.From<Pay>();
            queryPays.Where(o => o.BizType == typeof(Settlement).FullName && Sql.In(o.BizKvid, Kvids));
            var allPays = connF.Select(queryPays);
            if (!allPays.IsNullOrEmpty())
            {
                #region 先查出相关数据
                allToAccounts = connF.Select<Account>(o => Sql.In(o.Kvid, allPays.ConvertAll(p => p.ToKvid).Distinct()));
                var payFromPayments = allPays.Where(o => o.FromType == PayType.Payment).ToList();
                if (!payFromPayments.IsNullOrEmpty()) allPayments = connF.Select<Payment>(o => Sql.In(o.Kvid, payFromPayments.ConvertAll(p => p.FromKvid).Distinct()));
                var payFromAccounts = allPays.Where(o => Sql.In(o.FromType, PayType.Account, PayType.Discount)).ToList();
                if (!payFromAccounts.IsNullOrEmpty()) allFromAccounts = connF.Select<Account>(o => Sql.In(o.Kvid, payFromAccounts.ConvertAll(p => p.FromKvid).Distinct()));
                #endregion
            }
            foreach (var settlement in settlements)
            {
                if (!allPays.IsNullOrEmpty())
                {
                    var pays = allPays.Where(o => o.BizKvid == settlement.Kvid).ToList();
                    if (!pays.IsNullOrEmpty())
                    {
                        foreach (var pay in pays)
                        {
                            var toAccount = allToAccounts.FirstOrDefault(o => o.Kvid == pay.ToKvid);
                            if (toAccount == null) throw new Exception("未找到原收款账户!");
                            if (pay.FromType == PayType.Payment)
                            {
                                var payment = allPayments.FirstOrDefault(o => o.Kvid == pay.FromKvid);
                                if (payment != null)
                                {
                                    payment.Amount -= pay.Amount;
                                    payment.AddOnlyProperties(o => o.Amount);
                                    payment.AmountUsed -= pay.Amount;
                                    payment.AddOnlyProperties(o => o.AmountUsed);
                                    paymentsPreUpdate.Add(payment);

                                    var toDetail = new AccountDetail();
                                    toDetail.AccountKvid = toAccount.Kvid;
                                    toDetail.BizId = payment.SerialNumber;
                                    toDetail.BizKvid = payment.Kvid;
                                    toDetail.BizType = typeof(Payment).FullName;
                                    toDetail.PayerKvid = toAccount.Kvid;
                                    toDetail.PayerName = toAccount.Name;
                                    toDetail.PayerAccountKvid = toAccount.Kvid;
                                    toDetail.PayerAccountName = toAccount.Name;
                                    toDetail.PayerAccountSerialNumber = toAccount.SerialNumber;
                                    toDetail.PayeeKvid = payment.PayerKvid;
                                    toDetail.PayeeName = payment.PayerName;
                                    toDetail.AmountPayment = pay.Amount;
                                    toDetail.Summary = "结算项作废,分摊费用退回";
                                    accountDetailsPreInsert.Add(toDetail);
                                }
                            }
                            else
                            {
                                var fromAccount = allFromAccounts.FirstOrDefault(o => o.Kvid == pay.FromKvid);
                                if (fromAccount != null)
                                {
                                    var fromDetail = new AccountDetail();
                                    fromDetail.AccountKvid = fromAccount.Kvid;
                                    fromDetail.BizId = toAccount.SerialNumber;
                                    fromDetail.BizKvid = toAccount.Kvid;
                                    fromDetail.BizType = typeof(Account).FullName;
                                    fromDetail.PayerKvid = toAccount.Kvid;
                                    fromDetail.PayerName = toAccount.Name;
                                    fromDetail.PayerAccountKvid = toAccount.Kvid;
                                    fromDetail.PayerAccountName = toAccount.Name;
                                    fromDetail.PayerAccountSerialNumber = toAccount.SerialNumber;
                                    fromDetail.PayeeKvid = fromAccount.Kvid;
                                    fromDetail.PayeeName = fromAccount.Name;
                                    fromDetail.PayeeAccountKvid = fromAccount.Kvid;
                                    fromDetail.PayeeAccountName = fromAccount.Name;
                                    fromDetail.PayeeAccountSerialNumber = fromAccount.SerialNumber;
                                    fromDetail.AmountPayment = pay.Amount;
                                    fromDetail.Summary = "结算项作废,分摊费用退回";
                                    accountDetailsPreInsert.Add(fromDetail);

                                    var toDetail = new AccountDetail();
                                    toDetail.AccountKvid = toAccount.Kvid;
                                    toDetail.BizId = fromAccount.SerialNumber;
                                    toDetail.BizKvid = fromAccount.Kvid;
                                    toDetail.BizType = typeof(Account).FullName;
                                    toDetail.PayerKvid = toAccount.Kvid;
                                    toDetail.PayerName = toAccount.Name;
                                    toDetail.PayerAccountKvid = toAccount.Kvid;
                                    toDetail.PayerAccountName = toAccount.Name;
                                    toDetail.PayerAccountSerialNumber = toAccount.SerialNumber;
                                    toDetail.PayeeKvid = fromAccount.Kvid;
                                    toDetail.PayeeName = fromAccount.Name;
                                    toDetail.PayeeAccountKvid = fromAccount.Kvid;
                                    toDetail.PayeeAccountName = fromAccount.Name;
                                    toDetail.PayeeAccountSerialNumber = fromAccount.SerialNumber;
                                    toDetail.AmountPayment = pay.Amount;
                                    toDetail.Summary = "结算项作废,分摊费用退回";
                                    accountDetailsPreInsert.Add(toDetail);
                                }
                            }

                            var payOffset = pay.Offset("结算项作废");
                            paysOffsetPreUpdate.Add(pay);
                            paysOffsetPreInsert.Add(payOffset);
                        }
                    }
                }
                var settlementOffset = settlement.Offset("结算项作废");
                settlementsOffsetPreUpdate.Add(settlement);
                settlementsOffsetPreInsert.Add(settlementOffset);
            }
            if (!paymentsPreUpdate.IsNullOrEmpty())
            {
                parentsPreUpdate = connF.Select<Payment>(o => Sql.In(o.Kvid, paymentsPreUpdate.ConvertAll(p => p.ParentKvid).Distinct()));
            }
            var rtns = new RestfulExecutionResponse<Settlement>();
            rtns.Results = new List<Settlement>();

            var connE = KiviiContext.GetOpenedDbConnection<G>();
            #region  合并数据库连接,并启用事务
            IDbTransaction tranE = null, tranF = null;
            if (connE.ConnectionString == connF.ConnectionString)
            {
                connF.Close();
                connF = connE;
                tranE = connE.OpenTransaction();
            }
            else
            {
                tranE = connE.OpenTransaction();
                tranF = connF.OpenTransaction();
            }
            #endregion
            try
            {
                foreach (var payment in paymentsPreUpdate)
                {
                    connF.UpdateOnly(payment);
                }
                foreach (var item in parentsPreUpdate)
                {
                    item.RecalculateAmountUsed(connF);
                }
                foreach (var accountDetail in accountDetailsPreInsert)
                {
                    accountDetail.Insert(connF);
                }
                foreach (var pay in paysOffsetPreUpdate)
                {
                    connF.UpdateOnly(pay);
                }
                foreach (var pay in paysOffsetPreInsert)
                {
                    connF.Insert(pay);
                }
                foreach (var settlement in settlementsOffsetPreUpdate)
                {
                    connF.UpdateOnly(settlement);
                    rtns.Results.Add(settlement);
                    if (typeof(G) != typeof(Settlement))
                    {
                        var entitySettlement = new EntitySettlement<G>();
                        entitySettlement.PopulateInstance(settlement);
                        connE.UpdateOnly(entitySettlement);

                        #region 回写更新G数据
                        var instance = Activator.CreateInstance(typeof(G)) as IEntityHasSettlement;
                        if (instance != null)
                        {
                            instance.Kvid = settlement.BizKvid;
                            instance.AmountSettlement = settlement.Amount;
                            instance.AddOnlyProperties(o => o.AmountSettlement);
                            connE.UpdateOnly(instance);
                        }
                        #endregion
                    }
                }
                foreach (var item in settlementsOffsetPreInsert)
                {
                    connF.Insert(item);
                    if (typeof(G) != typeof(Settlement))
                    {
                        var entitySettlement = new EntitySettlement<G>();
                        entitySettlement.PopulateInstance(item);
                        connE.Insert(entitySettlement);
                    }
                }
                tranE?.Commit();
                tranF?.Commit();
            }
            catch (Exception ex)
            {
                tranE?.Rollback();
                tranF?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
